package Yaxx::ConfigFile;

use Hash::Merge;
use Config::General;
use IO::All;
use Class::MakeMethods::Standard::Hash ( 
					scalar => [ qw(required
						       type
						       name
						       path
						       file
						       parsed
						       priority
						      ) ]
				       );

1;

##****************************************************************************
sub new {
##****************************************************************************
    my $class = shift;
    my $self = { required => 1,
		 type     => '',
		 @_,
	       };
    bless $self, $class;

    # Force reading of config file
    $self->get_config;

    return $self;
}

##****************************************************************************
sub get_config {
##****************************************************************************
    my $self = $_[0];

    return $self->{config} if defined $self->{config};

    my @path = _listify($self->path);
    my $cfg_file_read;
    my %cfg;
    my $file;
    
    $self->parsed(0);
    for my $path (@path) {
	my $dir;
	eval { $dir = io($path); $dir->chdir };
	next if $@;		# Skip if failed to chdir to $path

	next unless -r ($file = $self->name);
	my $file_abs = io($file)->absolute->pathname;
	%cfg = ParseConfig(-ConfigFile => $file_abs,
			   -IncludeRelative => 1
			  );
	$self->file( $file_abs );
	$self->parsed(1);
	last;
    }

    die "ERROR - Unable to find required $self->{type} config file $self->{name} in ", join(":",@path), "\n"
      unless ($self->parsed or not $self->required);

    $self->{config} = \%cfg;

    return $self->{config};
}

##****************************************************************************
sub merge {
##****************************************************************************
    my $self = shift;
    my $config = shift;
    Hash::Merge::set_behavior( 'RIGHT_PRECEDENT' );
    return Hash::Merge::merge($config, $self->get_config);
}


########################################################################################
sub _listify {
########################################################################################
    if (defined $_[0]) {
	return ref($_[0]) eq 'ARRAY' ? @{$_[0]} : ($_[0]);
    } else {
	return ();
    }
}

